package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.AlarmRedisService;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AlarmRule;
import com.google.gson.Gson;
import com.sun.org.apache.regexp.internal.RE;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import retrofit2.http.POST;

import javax.print.DocFlavor;
import javax.print.attribute.standard.RequestingUserName;
import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/circuit")
public class CircuitController {

    private Logger logger = LoggerFactory.getLogger(CircuitController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private CircuitTransferService circuitTransferService;

    @Autowired
    private AlarmLogService alarmLogService;


    @Autowired
    private  BoschAlarmService boschAlarmService;

    @Autowired
    private AlarmRedisService alarmRedisService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private AlarmService alarmService;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private ConfigService configService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private InfoRedisService infoRedisService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private CarService carService;

    @Autowired
    private I18nController  i18n;

    @Autowired
    private TopicService topicService;

    @Autowired
    private TopicConfigService topicConfigService;

    @Value("${spring.influxdb.database}")
    private  String database ;

    //获取本公司的线路信息
    @ApiOperation(value = "获取本公司的线路信息",notes = "获取本公司的线路信息，返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号" +
            "   cargoName:货物别名" +
            "   startCity:起运城市," +
            "   startTime:起运时间," +
            "   endCity:目的城市," +
            "   endTime:目的时间," +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarmType:预警类型," +
            "   alarm:报警详情（1-温度，2-湿度，3-震动，4-倾角，5-延误，6-离线,7--在线）" +
            "   evaluate:运输评价" +
            "   transportation:运输方式" +
            "   plateNo:车牌号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(@RequestParam(value ="page",defaultValue = "1")Integer page,
                         @RequestParam(value = "rows",defaultValue = "5")Integer rows,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        List<CircuitResultVo>  list = circuitService.getAll(user.getCompanyNo(),(page-1)*rows,rows);
        for(int  i = 0,length=list.size();i<length;i++){
            if(list.get(i).getCircuitState() == 1){
                //获取设备在线情况
                Object data = infoRedisService.getHash("DeviceOnline",list.get(i).getgType()+"_"+list.get(i).getgSN());
                if(data == null){
                    if(StringUtils.isEmpty(list.get(i).getAlarm())){
                        list.get(i).setAlarm("6");
                    }else{
                        list.get(i).setAlarm(list.get(i).getAlarm()+",6");
                    }
                }else{
                    OnlineVo onlineVo =  gson.fromJson(data.toString(),OnlineVo.class);
                    if(onlineVo.getOnline() == 0){
                        if(StringUtils.isEmpty(list.get(i).getAlarm())){
                            list.get(i).setAlarm("6");
                        }else{
                            list.get(i).setAlarm(list.get(i).getAlarm()+",6");
                        }
                    }else if(onlineVo.getOnline() == 1) {
                        if(StringUtils.isEmpty(list.get(i).getAlarm())){
                            list.get(i).setAlarm("7");
                        }else{
                            list.get(i).setAlarm(list.get(i).getAlarm()+",7");
                        }
                    }
                }
            }else{
                continue;
            }
        }
        Integer num = circuitService.getAllCount(user.getCompanyNo());
        return ResponseEntity.ok(new EasyUIResult(num, list));
    }



    //根据运输编号查询本公司的运输信息（web端）
    @ApiOperation(value = "根据运输编号查询本公司的运输信息（web端）",notes = "根据运输编号查询本公司的运输信息（web端），返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号" +
            "   cargoName:货物别名" +
            "   startCity:起运城市," +
            "   startTime:起运时间," +
            "   endCity:目的城市," +
            "   endTime:目的时间," +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarmType:预警类型," +
            "   alarm:报警详情（比如：温度,湿度,震动）" +
            "   evaluate:运输评价")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByTransportationNo",method = RequestMethod.POST)
    public Object getByTransportationNo(@RequestBody String transportationNo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(transportationNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject=JSON.parseObject(transportationNo);
        transportationNo=(String) jsonObject.get("transportationNo");
        return circuitService.getByTransportationNo(transportationNo,user.getCompanyNo());

    }



    //根据运输编号查询本公司的运输信息（app端）
    @ApiOperation(value = "根据运输编号查询本公司的运输信息（app端）",notes = "根据运输编号查询本公司的运输信息（app端），返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoName:货物别名" +
            "   city:城市," +
            "   startTime:起运时间," +
            "   cargoNo:货物编号," +
            "   plateNo:车牌号," +
            "   deliveryAddress:交付地点," +
            "   transportation:运输方式," +
            "   transportationType:运输类型," +
            "   SN:gateway序列号" +
            "   TDLSN:TDLSN序列号" +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByTransportationNoForApp",method = RequestMethod.POST)
    public Object getByTransportationNoForApp(@RequestBody String transportationNo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(transportationNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject=JSON.parseObject(transportationNo);
        transportationNo=(String) jsonObject.get("transportationNo");
        ResultCircuitForAppVo resultCircuitForAppVo=circuitService.getByTransportationNoForApp(transportationNo,user.getCompanyNo());
        resultCircuitForAppVo.setStartTime(System.currentTimeMillis());
        return resultCircuitForAppVo;
    }



    //根据条件查询
    @ApiOperation(value = "根据条件查询本公司的线路信息",notes = "companyName不用传，根据条件查询:" +
            "   cargoNo:货物编号" +
            "   startTime:出发时间,（时间戳）" +
            "   endTime:目的时间,（时间戳）" +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   time:日期"+
            "返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号" +
            "   cargoName:货物别名" +
            "   startCity:起运城市," +
            "   startTime:起运时间," +
            "   endCity:目的城市," +
            "   endTime:目的时间," +
            "   alarmType:预警类型," +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarm:报警详情（比如：温度,湿度,震动）," +
            "   evaluate:运输评价")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByTerm",method = RequestMethod.POST)
    public Object getByTerm(@RequestBody CircuitTermVo circuitTermVo,@RequestParam(value ="page",defaultValue = "1")Integer page,
                            @RequestParam(value = "rows",defaultValue = "5")Integer rows,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(circuitTermVo.getCargoNo())){
            circuitTermVo.setCargoNo(null);
        }
        if(circuitTermVo.getCircuitState() == null || circuitTermVo.getCircuitState() == 3){
            circuitTermVo.setCircuitState(null);
        }
        if(circuitTermVo.getStartTime()==null){
            circuitTermVo.setStartTime(null);
        }
        if(circuitTermVo.getEndTime()==null){
            circuitTermVo.setEndTime(null);
        }
        if(circuitTermVo.getStartTime()!=null && circuitTermVo.getEndTime()!=null  && circuitTermVo.getStartTime()>circuitTermVo.getEndTime()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongTime"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(circuitTermVo.getTime())){
            circuitTermVo.setTime(null);
        }else{
            circuitTermVo.setTime(circuitTermVo.getTime().replace("天","").trim());
        }
        circuitTermVo.setCompanyNo(user.getCompanyNo());
        Integer num = circuitService.getByTermCount(circuitTermVo);
        List<CircuitResultVo> circuitResultVos =  circuitService.getByTerm(circuitTermVo,(page-1)*rows,rows);
        return ResponseEntity.ok(new EasyUIResult(num, circuitResultVos));
    }



    //根据线路编号获取线路详细信息
    @ApiOperation(value = "根据线路编号获取线路详细信息",notes = "根据线路编号获取线路详细信息，传值说明：" +
            "   transportationNo:运输编号；   " +
            "返回值说明：" +
            "   transportationNo：运输编号," +
            "   routeName:线路," +
            "   cargoName:货物别名," +
            "   transportation：运输方式," +
            "   evaluate：评价," +
            "   alarmType：预警类型," +
            "   SN:gateway序列号," +
            "   tdlList:{" +
            "       tdlsn：TDLSN序列号" +
            "       batteryVoltage:电池电压}," +
            "   list：{" +
            "       city：城市;" +
            "       time：到达时间/发车时间;" +
            "       last：运行时长;" +
            "       alarmCount：报警数;" +
            "       type：1--出发时间；2--目的时间}," +
            "   alarmLogList：{" +
            "       transportationNo：运输编号;" +
            "       TDLSN：tag序列号;" +
            "       description：报警描述;" +
            "       data：数值;" +
            "       alarmTime：报警时间;" +
            "       classify：分类}" +
            "   cityMessageList:{" +
            "       city：城市;" +
            "       lng：经度;" +
            "       lat：纬度;" +
            "       expTime：预计发车时间;" +
            "       comTime：预计到达时间" +
            "       type:1--出发地；2--目的地}" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getDetails")
    public Object getDetails(@RequestBody String transportationNo,HttpServletRequest request)  {
        if(StringUtils.isEmpty(transportationNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=(String)jsonObject.get("transportationNo");
        //获取线路信息
        CircuitMessage circuitVo = circuitService.getMessage(transportationNo);
        if(circuitVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return  gson.toJson(fb);
        }
        CircuitMessageVo messageVo =  new CircuitMessageVo(circuitVo.getTransportationNo(),circuitVo.getRouteName(),circuitVo.getCargoName(),circuitVo.getPlateNo(),circuitVo.getTransportation(),circuitVo.getEvaluate(),circuitVo.getAlarmType());
        //获取gateway信息
        TDLLogVo gatewayVo = tdlLogService.getByTransportationNo(transportationNo);
        List<String> TDL= tdlLogService.getTdlSN(transportationNo);
        List<ResultAlarmLog> alarmLogList = new ArrayList<>();
        List<CircuitMessageVo.TDL> tdlList = new ArrayList<>();
        List<ResultCityVo> cityVoList = new ArrayList<>();
        List<CircuitMessageVo.CityMessage>  cityMessageList =new ArrayList<>();
        if(gatewayVo == null || TDL.size() == 0){
            messageVo.setSN("NA");
            messageVo.setBattery("NA");
            messageVo.setAlarmLogList(alarmLogList);
            messageVo.setTdlList(tdlList);
            messageVo.setList(cityVoList);
            messageVo.setCityMessageList(cityMessageList);
            return gson.toJson(messageVo);
        }
        messageVo.setSN(gatewayVo.getgSN());
        alarmLogList= alarmLogService.getByTransportationNo(transportationNo);
        messageVo.setAlarmLogList(alarmLogList);
        String  sql ="";
        if (circuitVo.getEndTime() != null && circuitVo.getEndTime() != 0) {
            sql = "SELECT  \"battery\"  FROM \"tdl_policy\".\"" + gatewayVo.getgType() + "_" + gatewayVo.getgSN() + "\" where  time >=" + circuitVo.getStartTime() + "  and time<=" + circuitVo.getEndTime() * 1000000l + " ORDER BY time  desc limit 1";
        } else {
            sql = "SELECT \"battery\"   FROM \"tdl_policy\".\"" + gatewayVo.getgType() + "_" + gatewayVo.getgSN() + "\" where time >=" + circuitVo.getStartTime() + " ORDER BY time desc limit 1";
        }
        QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
        if (queryResult.getResults().get(0).getSeries() == null) {
            messageVo.setBattery("NA");
        }else{
            DecimalFormat df = new DecimalFormat("######0.0");
            String val = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1) == null ? "" : queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString();
            Boolean boo = Double.valueOf(val) <0;
            if(Double.valueOf(val) <0){
                messageVo.setBattery("NA");
            }else{
                messageVo.setBattery(df.format(Double.valueOf(val)));
            }
        }
        for(int i=0;i<TDL.size();i++){
            CircuitMessageVo.TDL tdl = new CircuitMessageVo.TDL();
            tdl.setTdlsn(TDL.get(i).replace("TDL-", "").trim());
            if(circuitVo.getStartTime() == null || circuitVo.getStartTime() == 0){
                tdl.setBatteryVoltage("NA");
            }else {
                String  sql1 ="";
                if (circuitVo.getEndTime() != null && circuitVo.getEndTime() != 0) {
                    sql1= "SELECT  \"b\"  FROM \"tdl_policy\".\"" + gatewayVo.getgType() + "_" + gatewayVo.getgSN() + "\" where \"tdl\"='" + TDL.get(i) + "'  and time >=" + circuitVo.getStartTime() + "  and time<=" + circuitVo.getEndTime() * 1000000l + " ORDER BY time  desc limit 1";
                } else {
                    sql1 = "SELECT \"b\"   FROM \"tdl_policy\".\"" + gatewayVo.getgType() + "_" + gatewayVo.getgSN() + "\" where \"tdl\"='" + TDL.get(i) + "'  and time >=" + circuitVo.getStartTime() + " ORDER BY time desc limit 1";
                }
                QueryResult queryResult1 = influxDBTemplate.query(new Query(sql1, database));
                if (queryResult1.getResults().get(0).getSeries() == null) {
                    tdl.setBatteryVoltage("NA");
                }else{
                    DecimalFormat df = new DecimalFormat("######0.0");
                    String value = queryResult1.getResults().get(0).getSeries().get(0).getValues().get(0).get(1) == null ? "" : queryResult1.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString();
                    if(Double.valueOf(value) <0){
                        tdl.setBatteryVoltage("NA");
                    }else{
                        tdl.setBatteryVoltage(df.format(Double.valueOf(value)));
                    }
                }
            }
            tdlList.add(tdl);
        }
        messageVo.setTdlList(tdlList);
        //获取线路情况
        List<CircuitTransferVo> list =circuitTransferService.getCity(transportationNo);
        if(list == null){
            messageVo.setList(cityVoList);
            messageVo.setCityMessageList(cityMessageList);
            return gson.toJson(messageVo);
        }

        for (int i = 0,length = list.size(); i < length; i++) {
            if (list.get(i).getSequence() == 1) {
                CircuitMessageVo.CityMessage  cityMessage = new CircuitMessageVo.CityMessage(list.get(i).getCity(),list.get(i).getLng(), list.get(i).getLat(),list.get(i).getExpTime(),null,1);
                cityMessageList.add(cityMessage);
                ResultCityVo cityVo = new ResultCityVo();
                cityVo.setCity(list.get(i).getCity());
                cityVo.setAddress(list.get(i).getAddress());
                cityVo.setType(1);
                if(list.get(i).getStartTime() == null || list.get(i).getStartTime() == 0 ){
                    cityVo.setTime(0l);
                    cityVo.setLast("0");
                    cityVo.setAlarmCount(0);
                }else{
                    cityVo.setTime(list.get(i).getStartTime());
                    if(list.get(i + 1).getArrivalTime() !=null && list.get(i + 1).getArrivalTime() !=0){
                        //根据时间和运输编号获取报警信息
                        Integer count = alarmLogService.getByTime(transportationNo, list.get(i).getStartTime(), list.get(i + 1).getArrivalTime());
                        cityVo.setAlarmCount(count);
                        cityVo.setLast(getMistiming(list.get(i + 1).getArrivalTime(),list.get(i).getStartTime(),request));
                    }else{
                        Integer count = alarmLogService.getByTime(transportationNo, list.get(i).getStartTime(), System.currentTimeMillis());
                        cityVo.setAlarmCount(count);
                        cityVo.setLast(getMistiming(System.currentTimeMillis(),list.get(i).getStartTime(),request));
                    }
                }
                cityVoList.add(cityVo);
            }else if(list.get(i).getSequence() == list.size()){
                CircuitMessageVo.CityMessage  cityMessage = new CircuitMessageVo.CityMessage(list.get(i).getCity(),list.get(i).getLng(), list.get(i).getLat(),null,list.get(i).getCompTime(),2);
                cityMessageList.add(cityMessage);
                ResultCityVo cityVo = new ResultCityVo();
                cityVo.setCity(list.get(i).getCity());
                cityVo.setAddress(list.get(i).getAddress());
                if(list.get(i).getArrivalTime() == null  || list.get(i).getArrivalTime() == 0){
                    cityVo.setTime(0l);
                }else {
                    cityVo.setTime(list.get(i).getArrivalTime());
                }
                cityVo.setType(2);
                cityVoList.add(cityVo);
            }else{
                ResultCityVo cityVo2 = new ResultCityVo();
                cityVo2.setCity(list.get(i).getCity());
                cityVo2.setAddress(list.get(i).getAddress());
                cityVo2.setType(2);
                if(list.get(i).getArrivalTime() == null ||list.get(i).getArrivalTime() ==0){
                    cityVo2.setTime(0l);
                    cityVo2.setAlarmCount(0);
                    cityVo2.setLast("0");
                    cityVoList.add(cityVo2);
                    ResultCityVo cityVo1 = new ResultCityVo();
                    cityVo1.setCity(list.get(i).getCity());
                    cityVo1.setAddress(list.get(i).getAddress());
                    cityVo1.setTime(0l);
                    cityVo1.setType(1);
                    cityVo1.setAlarmCount(0);
                    cityVo1.setLast("0");
                    cityVoList.add(cityVo1);
                }else{
                    cityVo2.setTime(list.get(i).getArrivalTime());
                    //根据时间和运输编号获取报警信息
                    if(list.get(i).getStartTime() == null ||list.get(i).getStartTime() ==0){
                        Integer count2 = alarmLogService.getByTime(transportationNo, list.get(i).getArrivalTime(), System.currentTimeMillis());
                        cityVo2.setAlarmCount(count2);
                        cityVo2.setLast(getMistiming(System.currentTimeMillis(),list.get(i).getArrivalTime(),request));
                        cityVoList.add(cityVo2);
                        ResultCityVo cityVo1 = new ResultCityVo();
                        cityVo1.setCity(list.get(i).getCity());
                        cityVo1.setAddress(list.get(i).getAddress());
                        cityVo1.setTime(0l);
                        cityVo1.setType(1);
                        cityVo1.setAlarmCount(0);
                        cityVo1.setLast("0");
                        cityVoList.add(cityVo1);
                    }else{
                        Integer count2 = alarmLogService.getByTime(transportationNo, list.get(i).getArrivalTime(), list.get(i).getStartTime());
                        cityVo2.setAlarmCount(count2);
                        cityVo2.setLast(getMistiming(list.get(i ).getStartTime(),list.get(i).getArrivalTime(),request));
                        cityVoList.add(cityVo2);
                        ResultCityVo cityVo1 = new ResultCityVo();
                        cityVo1.setCity(list.get(i).getCity());
                        cityVo1.setAddress(list.get(i).getAddress());
                        cityVo1.setTime(list.get(i).getStartTime());
                        cityVo1.setType(1);
                        if(list.get(i + 1).getArrivalTime() == null  ||list.get(i + 1).getArrivalTime() ==0){
                            Integer count = alarmLogService.getByTime(transportationNo, list.get(i).getStartTime(), System.currentTimeMillis());
                            cityVo1.setAlarmCount(count);
                            cityVo1.setLast(getMistiming(System.currentTimeMillis(),list.get(i).getStartTime(),request));
                            cityVoList.add(cityVo1);
                        }else {
                            Integer count = alarmLogService.getByTime(transportationNo, list.get(i).getStartTime(), list.get(i + 1).getArrivalTime());
                            cityVo1.setAlarmCount(count);
                            cityVo1.setLast(getMistiming(list.get(i + 1).getArrivalTime(),list.get(i).getStartTime(),request));
                            cityVoList.add(cityVo1);
                        }
                    }
                }
            }
        }
        messageVo.setCityMessageList(cityMessageList);
        messageVo.setList(cityVoList);
        return gson.toJson(messageVo);
    }



    //根据车牌号查线路
    @ApiOperation(value = "根据车牌号查线路---mobile",notes = "根据车牌号查线路，返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号" +
            "   cargoName:货物别名" +
            "   startCity:起运城市," +
            "   startTime:起运时间," +
            "   endCity:目的城市," +
            "   endTime:目的时间," +
            "   alarmType:预警类型," +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarm:报警详情（比如：温度,湿度,震动）" +
            "   evaluate:运输评价")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByPlateNo",method = RequestMethod.POST)
    public Object getByPlateNo(@RequestBody String plateNo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(plateNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"plateNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(plateNo);
        plateNo=(String)jsonObject.get("plateNo");
        return circuitService.getByPlateNo(plateNo,user.getCompanyNo());
    }


    //添加线路信息
    @ApiOperation(value = "添加线路信息",notes = "添加线路信息，" +
            "   startAddressVo: 线路集合，包含" +
            "   {" +
            "       country:国家" +
            "       city:城市" +
            "       addressDetail:详细地址" +
            "       lng:经度" +
            "       lat:纬度" +
            "       postCode:邮编"+
            "       expTime:预计开始时间,（传时间戳）" +
            "   }" +
            "   addressVoList: 线路集合，包含" +
            "   {" +
            "       country:国家" +
            "       city:城市" +
            "       addressDetail:详细地址" +
            "       lng:经度" +
            "       lat:纬度" +
            "       postCode:邮编"+
            "       expTime:预计开始时间,（传时间戳）" +
            "       compTime:预计结束时间,（传时间戳）" +
            "   }" +
            "   endAddressVo: 线路集合，包含" +
            "   {" +
            "       country:国家" +
            "       city:城市" +
            "       addressDetail:详细地址" +
            "       lng:经度" +
            "       lat:纬度" +
            "       postCode:邮编"+
            "       compTime:预计结束时间,（传时间戳）" +
            "   }" +
            "   SN:网关编号," +
            "   tdlAlarmList：   " +
            "   {" +
            "       TDLSN:" +
            "       tdl编号," +
            "       alarmType:报警类型," +
            "   }" +
            "   cargoNo:货物编号," +
            "   plateNo:车牌号," +
            "   cargoName:货物别名," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/addCircuit",method = RequestMethod.POST)
    public Object addCircuit(@RequestBody AddCircuitVo addCircuitVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        ResultCompanyVo companyVo =  companyService.getByCompanyNo(user.getCompanyNo());
        if(StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getCountry())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startCountry"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getCity())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startCity"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getAddressDetail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startAddressDetail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getLat())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startLat"));
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getStartAddressVo().getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongStartLat"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getLng())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startLng"));
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getStartAddressVo().getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[0-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongStartLng"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getPostCode())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startPostCode"));
            return gson.toJson(fb);
        }
        if (addCircuitVo.getStartAddressVo().getExpTime()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"starExpTime"));
            return gson.toJson(fb);
        }
        Long time = System.currentTimeMillis();
        String countryVar=addCircuitVo.getStartAddressVo().getCountry()+"&";
        String cityVar=addCircuitVo.getStartAddressVo().getCity()+ "&";
        String addressDetailVar=addCircuitVo.getStartAddressVo().getAddressDetail() +"&";
        String lngVar=addCircuitVo.getStartAddressVo().getLng()+"&";
        String latVar = addCircuitVo.getStartAddressVo().getLat()+"&";
        String postCodeVar = addCircuitVo.getStartAddressVo().getPostCode()+"&";
        String expTimeVar = addCircuitVo.getStartAddressVo().getExpTime()+"&";
        String compTimeVar = "0&";
        String arrivalTimeVar=time +"&";
        String startTimeVar=0 +"&";
        if(addCircuitVo.getAddressVoList() != null){
            if(addCircuitVo.getAddressVoList().size()>3){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"addressVoList"));
                return gson.toJson(fb);
            }
            for(int i =0,length=addCircuitVo.getAddressVoList().size();i<length;i++){
                if(StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getCountry())){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"addressVoCountry"));
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getCity())){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"addressVoCity"));
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getAddressDetail())){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"addressVoAddressDetail"));
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getLat())){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"addressVoLat"));
                    return gson.toJson(fb);
                }
                if(!addCircuitVo.getAddressVoList().get(i).getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"wrongAddressVoLat"));
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getLng())){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"addressVoLng"));
                    return gson.toJson(fb);
                }
                if(!addCircuitVo.getAddressVoList().get(i).getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[0-7]\\d)((\\.\\d{1,6})?)|180)$")) {
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"wrongaddressVoLng"));
                    return gson.toJson(fb);
                }
                if (StringUtils.isEmpty(addCircuitVo.getAddressVoList().get(i).getPostCode())){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"addressVoPostCode"));
                    return gson.toJson(fb);
                }
                if (addCircuitVo.getAddressVoList().get(i).getExpTime() == null){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"addressVoExpTime"));
                    return gson.toJson(fb);
                }
                if (addCircuitVo.getAddressVoList().get(i).getCompTime() == null){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"addressVoCompTime"));
                    return gson.toJson(fb);
                }
                if (addCircuitVo.getAddressVoList().get(i).getCompTime() <=addCircuitVo.getAddressVoList().get(i).getExpTime()){
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"wrongAddressVoCompTime"));
                    return gson.toJson(fb);
                }
                if(i<addCircuitVo.getAddressVoList().size()-1){
                    if (addCircuitVo.getAddressVoList().get(i).getCompTime() >=addCircuitVo.getAddressVoList().get(i +1).getExpTime()){
                        fb.setCode(0);
                        fb.setMessage(i18n.getMessage(request,"wrongAddressVoExpTime"));
                        return gson.toJson(fb);
                    }
                }
                countryVar=countryVar +addCircuitVo.getAddressVoList().get(i).getCountry() +"&";
                cityVar=cityVar +addCircuitVo.getAddressVoList().get(i).getCity() +"&";
                addressDetailVar =addressDetailVar +addCircuitVo.getAddressVoList().get(i).getAddressDetail() +"&";
                lngVar = lngVar + addCircuitVo.getAddressVoList().get(i).getLng() +"&";
                latVar = latVar + addCircuitVo.getAddressVoList().get(i).getLat() +"&";
                postCodeVar=postCodeVar + addCircuitVo.getAddressVoList().get(i).getPostCode() +"&";
                expTimeVar = expTimeVar +addCircuitVo.getAddressVoList().get(i).getExpTime() +"&";
                compTimeVar  = compTimeVar  +addCircuitVo.getAddressVoList().get(i).getCompTime() +"&";
                startTimeVar =startTimeVar + "0&";
                arrivalTimeVar = arrivalTimeVar +"0&";
            }
        }
        if(StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getCountry())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCountry"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getCity())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCity"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getAddressDetail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endAddressDetail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getLat())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endLat"));
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getEndAddressVo().getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongEndLat"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getLng())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endLng"));
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getEndAddressVo().getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[0-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongEndLng"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getPostCode())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endPostCode"));
            return gson.toJson(fb);
        }
        if (addCircuitVo.getEndAddressVo().getCompTime()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCompTime"));
            return gson.toJson(fb);
        }
        if(addCircuitVo.getEndAddressVo().getCompTime()<=addCircuitVo.getStartAddressVo().getExpTime()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongEndCompTime"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gateway"));
            return gson.toJson(fb);
        }
        countryVar=countryVar +addCircuitVo.getEndAddressVo().getCountry() +"&";
        cityVar=cityVar +addCircuitVo.getEndAddressVo().getCity() +"&";
        addressDetailVar =addressDetailVar +addCircuitVo.getEndAddressVo().getAddressDetail() +"&";
        lngVar = lngVar + addCircuitVo.getEndAddressVo().getLng() +"&";
        latVar = latVar + addCircuitVo.getEndAddressVo().getLat() +"&";
        postCodeVar=postCodeVar + addCircuitVo.getEndAddressVo().getPostCode() +"&";
        expTimeVar = expTimeVar +"0&";
        compTimeVar  = compTimeVar  +addCircuitVo.getEndAddressVo().getCompTime() +"&";
        startTimeVar =startTimeVar + "0&";
        arrivalTimeVar = arrivalTimeVar +"0&";
        String type =gatewayService.getBySN(addCircuitVo.getSN(),user.getCompanyNo());
        //判断当前gateWay是否存在
        if(StringUtils.isEmpty(type)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayExist"));
            return gson.toJson(fb);
        }
        ResultGatewayVo resultGatewayVo = gatewayService.getBySNAndType(addCircuitVo.getSN(),type);
        if(!resultGatewayVo.getState()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayMounted"));
            return gson.toJson(fb);
        }
        //获取gateway的场景
//        if(resultGatewayVo.getUseScene()== 2){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"gatewayConsistent"));
//            return gson.toJson(fb);
//        }
        if(resultGatewayVo.getUseScene() != 0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"usedgateway"));
            return gson.toJson(fb);
        }
        String alarmTypeVar="";
        String TDLSNVar="";
        List<String> devList = tdlDeviceService.getByGatewaySN(addCircuitVo.getSN(),type);
        List<Float> tempL = new ArrayList<>();
        List<Float> tempH = new ArrayList<>();
        List<Float> humiL = new ArrayList<>();
        List<Float> humiH = new ArrayList<>();
        List<Integer> pressL = new ArrayList<>();
        List<Integer> pressH = new ArrayList<>();
        List<Integer> sugEnergy = new ArrayList<>();
        List<Float> accThreshold = new ArrayList<>();
        List<Integer> timeThreshold = new ArrayList<>();
        List<Integer> tiltThreshold = new ArrayList<>();
        for(int j = 0,length =addCircuitVo.getTdlAlarmList().size();j<length;j++){
            if(StringUtils.isEmpty(addCircuitVo.getTdlAlarmList().get(j).getTDLSN())){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"tdl"));
                return gson.toJson(fb);
            }
            //判断当前tdl是否存在
            ResultTDLDeviceVo tdlDeviceVo = tdlDeviceService.getByTDLSN(addCircuitVo.getTdlAlarmList().get(j).getTDLSN());
            if(tdlDeviceVo == null){
                fb.setCode(0);
                Object[]  args ={addCircuitVo.getTdlAlarmList().get(j).getTDLSN()};
                fb.setMessage(i18n.getMessage(request, args,"tdlExist"));
                return gson.toJson(fb);
            }
            if(!StringUtils.isEmpty(tdlDeviceVo.getGatewaySN())){
                fb.setCode(0);
                Object[]  args ={addCircuitVo.getTdlAlarmList().get(j).getTDLSN()};
                fb.setMessage(i18n.getMessage(request,args,"tdlUsed"));
                return gson.toJson(fb);
            }
            if(StringUtils.isEmpty(addCircuitVo.getTdlAlarmList().get(j).getAlarmType())){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"alarmType"));
                return gson.toJson(fb);
            }
            //获取阈值信息
            ResultAlarmVo resultAlarmVo =   alarmService.getByType(addCircuitVo.getTdlAlarmList().get(j).getAlarmType(),user.getCompanyNo());
            tempL.add(resultAlarmVo.getTemMin());
            tempH.add(resultAlarmVo.getTemMax());
            humiL.add(resultAlarmVo.getHumidityMin());
            humiH.add(resultAlarmVo.getHumidityMax());
            pressL.add(255);
            pressH.add(255);
            sugEnergy.add(1);
            accThreshold.add(Float.valueOf(resultAlarmVo.getShock()));
            timeThreshold.add(5);
            tiltThreshold.add((int)Double.parseDouble(resultAlarmVo.getTilt().toString()));
            alarmTypeVar =alarmTypeVar + addCircuitVo.getTdlAlarmList().get(j).getAlarmType() +"&";
            TDLSNVar =TDLSNVar + addCircuitVo.getTdlAlarmList().get(j).getTDLSN() +"&";
            devList.add("TDL-"+addCircuitVo.getTdlAlarmList().get(j).getTDLSN());
            AlarmRule.saveAlarmRule("TDL/"+type+"/" +addCircuitVo.getSN()+"/Data",addCircuitVo.getTdlAlarmList().get(j).getTDLSN(),addCircuitVo.getTdlAlarmList().get(j).getAlarmType(),user.getCompanyNo(),boschAlarmService,alarmRedisService);
        }
        if(StringUtils.isEmpty(addCircuitVo.getCargoNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"cargoNo"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getPlateNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"plateNo"));
            return gson.toJson(fb);
        }
        //判断当前车牌是否已存在
        if(carService.getByCarNo(user.getCompanyNo(),addCircuitVo.getPlateNo()) == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"nonexistentTruck"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getCargoName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"cargoName"));
            return gson.toJson(fb);
        }
        //根据sn获取type
        ConfigCMDVo configCMDVo = new ConfigCMDVo("config",5,15,devList,tempL,tempH,humiL,humiH,pressL,pressH,sugEnergy,accThreshold,timeThreshold,tiltThreshold,1,System.currentTimeMillis()/1000l);
        Map<Object,Object> map=new HashMap<>();
        String transportation = "公路";
        String transportationType="本地运输";
        String transportationNo =getTransportNo(transportation,transportationType);
        map.put("transportationNo",transportationNo);
        map.put("startTime",time);
        map.put("expTime",addCircuitVo.getStartAddressVo().getExpTime());
        map.put("compTime",addCircuitVo.getEndAddressVo().getCompTime());
        map.put("cargoNo",addCircuitVo.getCargoNo());
        map.put("cargoName",addCircuitVo.getCargoName());
        map.put("plateNo",addCircuitVo.getPlateNo());
        map.put("deliveryAddress","站点");
        map.put("transportation",transportation);
        map.put("transportationType",transportationType);
        map.put("createTime",time);
        map.put("remark",addCircuitVo.getRemark());
        map.put("counts",addCircuitVo.getAddressVoList()== null ?  2 : addCircuitVo.getAddressVoList().size()+2);
        map.put("companyNo",user.getCompanyNo());
        map.put("countryVar",countryVar);
        map.put("cityVar",cityVar);
        map.put("addressDetailVar",addressDetailVar);
        map.put("lngVar",lngVar);
        map.put("latVar",latVar);
        map.put("postCodeVar",postCodeVar);
        map.put("arrivalTimeVar",arrivalTimeVar);
        map.put("startTimeVar",startTimeVar);
        map.put("expTimeVar",expTimeVar);
        map.put("compTimeVar",compTimeVar);
        map.put("SN",addCircuitVo.getSN());
        map.put("type",type);
        map.put("TDLSNVar",TDLSNVar);
        map.put("alarmTypeVar",alarmTypeVar);
        map.put("TDLCounts",addCircuitVo.getTdlAlarmList()== null ?  0 : addCircuitVo.getTdlAlarmList().size());
        map.put("message",configCMDVo == null ? "" :gson.toJson(configCMDVo));
        map.put("flag",false);
        map.put("untie",false);
        circuitService.addCircuit(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("添加线路成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"circuitSuccess"));
            //增加配置（公司以及线路信息）
            AlarmMqttConfig alarmMqttConfig = new AlarmMqttConfig(transportationNo,companyVo.getCompanyNo());
            infoRedisService.hmSet("AlarmMqttConfig",type+"_"+addCircuitVo.getSN(),gson.toJson(alarmMqttConfig));
            List<ResultTopicVo> topicList = topicService.getByGateway(new GatewaySNAndTypeVo(addCircuitVo.getSN(),type));
            for (ResultTopicVo topic : topicList) {
                infoRedisService.delHashKey("TopicConfig", topic.getTopicName());
                //配置发布还是订阅
                ProbesConfigVo channel_config = new ProbesConfigVo();
                //根据设备的主题名，获取所有的通道
                List<ResultTopicConfigVo> devChannelList = topicConfigService.getProbes(topic.getTopicName(),addCircuitVo.getSN(),type);
                List<ChannelConfigVo> channelConfigs = new ArrayList<>();
                for (ResultTopicConfigVo devChannel : devChannelList) {
                    ChannelConfigVo channelConfig = new ChannelConfigVo(devChannel.getPort(),devChannel.getUnti(),devChannel.getDescription(),devChannel.getAlias());
                    channelConfigs.add(channelConfig);
                }
                channel_config.setIntervalTime(topic.getIntervalTime());
                channel_config.setChannelConfigs(channelConfigs);
                infoRedisService.hmSet("TopicConfig", topic.getTopicName(), channel_config.toString());
            }
        }else {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"circuitFailure"));
        }
        return gson.toJson(fb);
    }


    @ApiOperation(value = "添加线路信息----mobile",notes = "添加线路信息，" +
            "   cargoNo:货物编号," +
            "   plateNo:车牌号," +
            "   cargoName:货物别名," +
            "   transportation:运输方式（公路，铁路，海运，空运，快递）," +
            "   startAddressVo: 启运城市，包含" +
            "   {" +
            "       country:国家," +
            "       city:城市," +
            "       addressDetail:详细地址," +
            "       lng:经度," +
            "       lat:纬度," +
            "       postCode:邮编,"+
            "       expTime:预计开始时间（13位时间戳）" +
            "   }" +
            "   endAddressVo: 目的城市，包含" +
            "   {" +
            "       country:国家," +
            "       city:城市," +
            "       addressDetail:详细地址," +
            "       lng:经度," +
            "       lat:纬度," +
            "       postCode:邮编,"+
            "       compTime:预计结束时间（13位时间戳）" +
            "   }" +
            "   deliveryAddress：交付地点,(站点，仓库，港口)" +
            "   transportationType：运输类型,（本地运输，国际运输）," +
            "   remark:备注,")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/add",method = RequestMethod.POST)
    public Object add(@RequestBody AddCircuit addCircuitVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(addCircuitVo.getCargoNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"cargoNo"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getPlateNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"plateNo"));
            return gson.toJson(fb);
        }
        //判断当前车牌是否已存在
        if(carService.getByCarNo(user.getCompanyNo(),addCircuitVo.getPlateNo()) == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"nonexistentTruck"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getCargoName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"cargoName"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getTransportation())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportation"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getCountry())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startCountry"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getCity())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startCity"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getAddressDetail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startAddressDetail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getLat())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startLat"));
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getStartAddressVo().getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongStartLat"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getLng())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startLng"));
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getStartAddressVo().getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[0-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongStartLng"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getStartAddressVo().getPostCode())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startPostCode"));
            return gson.toJson(fb);
        }
        if (addCircuitVo.getStartAddressVo().getExpTime()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"starExpTime"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getCountry())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCountry"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getCity())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCity"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getAddressDetail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endAddressDetail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getLat())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endLat"));
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getEndAddressVo().getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongEndLat"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getLng())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endLng"));
            return gson.toJson(fb);
        }
        if(!addCircuitVo.getEndAddressVo().getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[0-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongEndLng"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitVo.getEndAddressVo().getPostCode())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endPostCode"));
            return gson.toJson(fb);
        }
        if (addCircuitVo.getEndAddressVo().getCompTime()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCompTime"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getDeliveryAddress())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deliveryAddress"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(addCircuitVo.getTransportationType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationType"));
            return gson.toJson(fb);
        }
        Long time = System.currentTimeMillis();
        String countryVar=addCircuitVo.getStartAddressVo().getCountry()+"&"+addCircuitVo.getEndAddressVo().getCountry() +"&";
        String cityVar=addCircuitVo.getStartAddressVo().getCity()+ "&"+addCircuitVo.getEndAddressVo().getCity() +"&";
        String addressDetailVar=addCircuitVo.getStartAddressVo().getAddressDetail() +"&"+addCircuitVo.getEndAddressVo().getAddressDetail() +"&";
        String lngVar=addCircuitVo.getStartAddressVo().getLng()+"&"+ addCircuitVo.getEndAddressVo().getLng() +"&";
        String latVar = addCircuitVo.getStartAddressVo().getLat()+"&"+ addCircuitVo.getEndAddressVo().getLat() +"&";
        String postCodeVar = addCircuitVo.getStartAddressVo().getPostCode()+"&"+ addCircuitVo.getEndAddressVo().getPostCode() +"&";
        String expTimeVar = addCircuitVo.getStartAddressVo().getExpTime()+"&"+"0 ";
        String compTimeVar = "0&"+addCircuitVo.getEndAddressVo().getCompTime() +"&";
        String arrivalTimeVar="0&0&";
        String startTimeVar="0&0&";
        //根据sn获取type
        Map<Object,Object> map=new HashMap<>();
        map.put("transportationNo",getTransportNo(addCircuitVo.getTransportation(),addCircuitVo.getTransportationType()));
        map.put("expTime",addCircuitVo.getStartAddressVo().getExpTime());
        map.put("compTime",addCircuitVo.getEndAddressVo().getCompTime());
        map.put("cargoNo",addCircuitVo.getCargoNo());
        map.put("cargoName",addCircuitVo.getCargoName());
        map.put("plateNo",addCircuitVo.getPlateNo());
        map.put("deliveryAddress",addCircuitVo.getDeliveryAddress());
        map.put("transportation",addCircuitVo.getTransportation());
        map.put("transportationType",addCircuitVo.getTransportationType());
        map.put("createTime",time);
        map.put("remark",addCircuitVo.getRemark());
        map.put("companyNo",user.getCompanyNo());
        map.put("counts",2);
        map.put("countryVar",countryVar);
        map.put("cityVar",cityVar);
        map.put("addressDetailVar",addressDetailVar);
        map.put("lngVar",lngVar);
        map.put("latVar",latVar);
        map.put("postCodeVar",postCodeVar);
        map.put("arrivalTimeVar",arrivalTimeVar);
        map.put("startTimeVar",startTimeVar);
        map.put("expTimeVar",expTimeVar);
        map.put("compTimeVar",compTimeVar);
        circuitService.add(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("线路添加成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"circuitSuccess"));
        }else {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"circuitFailure"));
        }
        return gson.toJson(fb);
    }



    //修改线路结束时间
    @ApiOperation(value = "修改线路结束时间",notes = "修改线路结束时间，传值说明：" +
            "   transportationNo:运输编号," +
            "   endTime:到达时间 " )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateEndTime",method = RequestMethod.POST)
    public Object updateEndTime(@RequestBody UpdateCircuitVo updateCircuitVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(updateCircuitVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        if (circuitService.getByTransportationNo(updateCircuitVo.getTransportationNo(),user.getCompanyNo())==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if (updateCircuitVo.getEndTime()==null){
            updateCircuitVo.setEndTime(System.currentTimeMillis());
        }
        if (circuitService.getByTransportationNo(updateCircuitVo.getTransportationNo(),user.getCompanyNo()).getCircuitState() ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notStarted"));
            return gson.toJson(fb);
        }
        ResultCircuitVo resultCircuitVo =  circuitService.getByTransportationNo(updateCircuitVo.getTransportationNo(),user.getCompanyNo());
        //获取当前线路的gateway 和tdl信息
        TDLLogVo tdlLogVo = tdlLogService.getByTransportationNo(updateCircuitVo.getTransportationNo());
        if(tdlLogVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        List<String> dev = tdlLogService.getTdlSN(updateCircuitVo.getTransportationNo());
        if(dev == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        List<String> devList = tdlDeviceService.getByGatewaySN(tdlLogVo.getgSN(),tdlLogVo.getgType());
        if(devList == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        String tdl =  tdlDeviceService.getTdl(tdlLogVo.getgSN(),tdlLogVo.getgType());
        if(tdl == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        ConfigCMDVo configCMDVo;
        ConfigVo configVo =configService.getConfig(tdlLogVo.getgSN(),tdlLogVo.getgType());
        if(configVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(configVo.getMessage())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        ConfigCMDVo config = gson.fromJson(configVo.getMessage(),ConfigCMDVo.class);
        Boolean untie = false;
        if(tdl.equals(tdlLogVo.getTDLSN())){
            configCMDVo =null;
            untie = true;
        }else{
            devList.removeAll(dev);
            configCMDVo = new ConfigCMDVo("config",config.getGpsPeriod(),config.getGprsPeriod(),devList,config.getMode(),System.currentTimeMillis()/1000l);
        }

        Map<Object,Object> map=new HashMap<>();
        map.put("transportationNo",updateCircuitVo.getTransportationNo());
        map.put("gSN",tdlLogVo.getgSN());
        map.put("gType",tdlLogVo.getgType());
        map.put("startCity",resultCircuitVo.getStartCity());
        map.put("endCity",resultCircuitVo.getEndCity());
        map.put("message",configCMDVo == null ? "" : gson.toJson(configCMDVo));
        map.put("endTime",updateCircuitVo.getEndTime());
        map.put("untie",untie);
        circuitService.endCircuit(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("修改成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"endCircuitSuccess"));
            String key ="TDL/"+tdlLogVo.getgType()+"/" +tdlLogVo.getgSN() +"/Data";
            alarmRedisService.delKey(key);
            infoRedisService.delHashKey("AlarmMqttConfig",tdlLogVo.getgType()+"_"+tdlLogVo.getgSN());
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCircuitFailure"));
        }
        return gson.toJson(fb);
    }



    //评价
    @ApiOperation(value = "评价---mobile",notes = "评价线路传值说明：" +
            "   transportationNo:运输编号," +
            "   evaluate:评分，" +
            "   evaluateRemark:评分备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateEvaluate",method = RequestMethod.POST)
    public Object updateEvaluate(@RequestBody UpdateEvaluateVo updateEvaluateVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(updateEvaluateVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        if (circuitService.getByTransportationNo(updateEvaluateVo.getTransportationNo(),user.getCompanyNo())==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if (updateEvaluateVo.getEvaluate()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"evaluate"));
            return gson.toJson(fb);
        }

        if (circuitService.getByTransportationNo(updateEvaluateVo.getTransportationNo(),user.getCompanyNo()).getCircuitState()!=2){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"completedCircuit"));
            return gson.toJson(fb);
        }
        int a = circuitService.updateEvaluate(updateEvaluateVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"evaluateSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"evaluateFailure"));
        }
        return gson.toJson(fb);
    }



    //根据运输状态查询线路信息
    @ApiOperation(value = "根据运输状态查询线路信息---mobile",notes = "根据运输状态查询线路信息，传值说明：{\"circuitState\":2}  ,circuitState:线路状态（0未开始，1运输中，2完成），返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号" +
            "   cargoName:货物别名" +
            "   startCity:起运城市," +
            "   startTime:起运时间," +
            "   endCity:目的城市," +
            "   endTime:目的时间," +
            "   alarmType:预警类型," +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarm:报警详情（比如：温度,湿度,震动）" +
            "   evaluate:运输评价" +
            "   transportation:运输方式")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByCircuitState",method = RequestMethod.POST)
    public Object getByCircuitState(@RequestBody Map map,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (!map.containsKey("circuitState")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"circuitState"));
            return gson.toJson(fb);
        }
        return circuitService.getByCircuitState(Integer.valueOf(map.get("circuitState").toString()),user.getCompanyNo());
    }


    //删除线路
    @ApiOperation(value = "删除线路",notes = "删除线路，传值说明：" +
            "   transportationNo:运输编号" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/delCircuit",method = RequestMethod.POST)
    public Object delCircuit(@RequestBody String transportationNo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=(String)jsonObject.get("transportationNo");
        ResultCircuitVo resultCircuitVo =  circuitService.getByTransportationNo(transportationNo,user.getCompanyNo());
        if (resultCircuitVo==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if (resultCircuitVo.getCircuitState()==1){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"inTransit"));
            return gson.toJson(fb);
        }
        TDLLogVo tdlLogVo =  tdlLogService.getByTransportationNo(transportationNo);
        if(tdlLogVo != null &&  resultCircuitVo.getCircuitState() == 0){
            List<String> dev = tdlLogService.getTdlSN(transportationNo);
            if(dev == null){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"bindTDL"));
                return gson.toJson(fb);
            }
            List<String> devList = tdlDeviceService.getByGatewaySN(tdlLogVo.getgSN(),tdlLogVo.getgType());
            if(dev == null){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"bindTDL"));
                return gson.toJson(fb);
            }
            String tdl =  tdlDeviceService.getTdl(tdlLogVo.getgSN(),tdlLogVo.getgType());
            if(StringUtils.isEmpty(tdl)){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"bindTDL"));
                return gson.toJson(fb);
            }
            ConfigCMDVo configCMDVo;
            ConfigVo configVo =configService.getConfig(tdlLogVo.getgSN(),tdlLogVo.getgType());
            if(configVo ==null ){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"bindGateway"));
                return gson.toJson(fb);
            }
            if(StringUtils.isEmpty(configVo.getMessage())){
                configCMDVo = new ConfigCMDVo();
            }else{
                ConfigCMDVo config = gson.fromJson(configVo.getMessage(),ConfigCMDVo.class);
                if(tdl.equals(tdlLogVo.getTDLSN())){
                    configCMDVo = new ConfigCMDVo();
                }else{
                    devList.removeAll(dev);
                    configCMDVo = new ConfigCMDVo("config",config.getGpsPeriod(),config.getGprsPeriod(),devList,config.getMode(),System.currentTimeMillis()/1000l);
                }
            }
            //解绑gateway和tdl
            Map<Object,Object> map=new HashMap<>();
            map.put("transportationNo",transportationNo);
            map.put("gSN",tdlLogVo.getgSN());
            map.put("gType",tdlLogVo.getgType());
            map.put("endTime",System.currentTimeMillis());
            map.put("message",gson.toJson(configCMDVo));
            circuitService.deleteCircuit(map);
            Map<Object,Object> msg=new HashMap<>();
            msg.put("msg",map.get("msg"));
            if (msg.get("msg").equals("解除网关成功")){
                fb.setCode(1);
                fb.setMessage(i18n.getMessage(request,"circuitDeleteSuccess"));
                String key ="TDL/"+tdlLogVo.getgType()+"/" +tdlLogVo.getgSN() +"/Data";
                alarmRedisService.delKey(key);
                infoRedisService.delHashKey("AlarmMqttConfig",tdlLogVo.getgType()+"_"+tdlLogVo.getgSN());
            }else{
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"circuitDeleteFailure"));
            }
        }else {
            int a = circuitService.delCircuit(transportationNo);
            try {
                if (a > 0) {
                    fb.setCode(1);
                    fb.setMessage(i18n.getMessage(request,"circuitDeleteSuccess"));
                    infoRedisService.delHashKey("AlarmMqttConfig",tdlLogVo.getgType()+"_"+tdlLogVo.getgSN());
                } else {
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"circuitDeleteFailure"));
                }
            } catch (Exception e){
                e.printStackTrace();
                logger.info(e.toString());
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"circuitDeleteFailure"));
            }
        }
        return gson.toJson(fb);
    }


    //开始运输
    @ApiOperation(value = "开始运输",notes = "开始运输" +
            "startTime:开始时间（13位时间戳）" +
            "transportationNo:运输编号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/startTransportation")
    public  Object  startTransportation(@RequestBody StartCircuitVo startCircuitVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(startCircuitVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        if (circuitService.getByTransportationNo(startCircuitVo.getTransportationNo(),user.getCompanyNo())==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if (startCircuitVo.getStartTime()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startTime"));
            return gson.toJson(fb);
        }
        //判断该线路是否绑定
        ResultGatewayVo gatewayVo = tdlLogService.getByNo(startCircuitVo.getTransportationNo());
        if(gatewayVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("transportationNo",startCircuitVo.getTransportationNo());
        map.put("startTime",startCircuitVo.getStartTime());
        circuitService.updateStartTime(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("修改成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"startCircuitSuccess"));
        }else {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startCircuitFailure"));
        }
        return gson.toJson(fb);
    }



    //判断线路是否已经绑定了tdl和gateway
    @ApiOperation(value = "判断线路是否已经绑定了tdl和gateway",notes = "判断线路是否已经绑定了tdl和gateway" +
            "transportationNo:运输编号" +
            "返回值：" +
            "binding：false未绑定/true绑定了")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/circuitbBinding")
    public Object circuitbBinding(@RequestBody String transportationNo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=(String)jsonObject.get("transportationNo");
        ResultCircuitVo resultCircuitVo =  circuitService.getByTransportationNo(transportationNo,user.getCompanyNo());
        if (resultCircuitVo==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        ResultTDLDeviceVo tdlDeviceVo =  tdlDeviceService.getByTransportationNo(transportationNo);
        BindVo bindVo = new BindVo();
        if(tdlDeviceVo == null){
            bindVo.setBinding(false);
        }else{
            bindVo.setBinding(true);
        }
        return gson.toJson(bindVo);
    }

    //获取正在未结束的线路时间配置
    @ApiOperation(value = "获取未结束的线路配置间隔时间",notes = "获取未结束的线路配置间隔时间" +
            "transportationNo:运输编号" +
            "返回值：" +
            "gps:定位间隔" +
            "gprs:发送间隔")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/queryConfig")
    public Object queryConfig(@RequestBody String transportationNo,HttpServletRequest request){
        UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=(String)jsonObject.get("transportationNo");
        ResultCircuitVo resultCircuitVo =  circuitService.getByTransportationNo(transportationNo,user.getCompanyNo());
        if (resultCircuitVo==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if(resultCircuitVo.getCircuitState()==2){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCircuit"));
            return gson.toJson(fb);
        }
        //获取相关网关数据已经配置数据
        ConfigResultVo configResultVo =  configService.getByTransportationNo(transportationNo);
        if(configResultVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCircuit"));
            return gson.toJson(fb);
        }
        CheckInVo checkInVo = new Gson().fromJson(configResultVo.getMessage(),CheckInVo.class);
        return gson.toJson(new GPSVo(transportationNo,checkInVo.getGpsPeriod(),checkInVo.getGprsPeriod()));
    }

    //修改某一个条线路的定位间隔和时间间隔
    @ApiOperation(value = "修改某一个条线路的定位间隔和时间间隔",notes = "修改某一个条线路的定位间隔和时间间隔" +
            "transportationNo:运输编号" +
            "gps:定位间隔" +
            "gprs:发送间隔")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/updateConfig")
    public Object updateConfig(@RequestBody GPSVo gpsVo,HttpServletRequest request){
        UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        if (StringUtils.isEmpty(gpsVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        if (gpsVo.getGps() == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gpsPeriod"));
            return gson.toJson(fb);
        }
        if (gpsVo.getGprs() == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gprsPeriod"));
            return gson.toJson(fb);
        }
        ResultCircuitVo resultCircuitVo =  circuitService.getByTransportationNo(gpsVo.getTransportationNo(),user.getCompanyNo());
        if (resultCircuitVo==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if(resultCircuitVo.getCircuitState()==2){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCircuit"));
            return gson.toJson(fb);
        }
        ConfigResultVo configResultVo =  configService.getByTransportationNo(gpsVo.getTransportationNo());
        if(configResultVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCircuit"));
            return gson.toJson(fb);
        }
        CheckInVo checkInVo = new Gson().fromJson(configResultVo.getMessage(),CheckInVo.class);
        checkInVo.setGprsPeriod(gpsVo.getGprs());
        checkInVo.setGpsPeriod(gpsVo.getGps());
        configResultVo.setMessage(gson.toJson(checkInVo));
        Integer a = configService.updateGPS(configResultVo);
        if(a == 1){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"updateconfigSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"updateconfigFailure"));
        }
        return  gson.toJson(fb);

    }




    //生成运输编号
    public String getTransportNo(String transportation,String transportationType){
        String transportationNo="";
        if(transportationType.equals("本地运输")){
            transportationNo = transportationNo+"CHN";
        }else{
            transportationNo = transportationNo+"HUN";
        }
        transportationNo=transportationNo+new SimpleDateFormat("yyyyMMdd").format(new Date());
        if(transportation.equals("公路")){
            transportationNo = transportationNo +"G";
        } else if (transportation.equals("铁路")){
            transportationNo = transportationNo +"T";
        }else if (transportation.equals("空运")){
            transportationNo = transportationNo +"K";
        }else if (transportation.equals("快递")){
            transportationNo = transportationNo +"D";
        }else if (transportation.equals("海运")){
            transportationNo = transportationNo +"H";
        }
        transportationNo=transportationNo+((int)(Math.random()*(9999-1000+1))+1000);
        if (circuitService.getNo(transportationNo)>0){
            while (circuitService.getNo(transportationNo)>0){
                transportationNo=transportationNo+((int)(Math.random()*(9999-1000+1))+1000);
            }
        }
        return transportationNo;
    }



    //两个时间戳之间相差多少
    public String getMistiming(Long startTime,Long arrivalTime,HttpServletRequest request){
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = startTime- arrivalTime;
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        long sec = diff % nd % nh % nm / ns;
        return  day + i18n.getMessage(request,"day")+" " + hour + i18n.getMessage(request,"hour") +" " + min +i18n.getMessage(request,"min") ;
    }




    //    修改线路信息
//    @ApiOperation(value = "修改线路信息",notes = "修改线路信息，传值说明：" +
//            "   transportationNo:运输编号（自动生成，作为修改条件）" +
//            "   addressVoList: 线路集合，包含" +
//            "{sequence:顺序（出发地0，目的地从1依次排序）" +
//            "country:国家" +
//            "city:城市" +
//            "addressDetail:详细地址" +
//            "lng:经度" +
//            "lat:纬度" +
//            "postCode:邮编"+
//            "expTime:预计开始时间,（传时间戳）" +
//            "compTime:预计结束时间,（传时间戳）}" +
//            "   gSN:网关编号," +
//            "   TDLSN:tdl编号," +
//            "   alarmType:报警类型," +
//            "   cargoNo:货物编号," +
//            "   plateNo:车牌号," +
//            "   cargoName:货物别名," +
//            "   remark:备注")
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    @RequestMapping(value="/updateCircuit",method = RequestMethod.POST)
//    public Object updateCircuit(@RequestBody UpdateCircuitInfo updateCircuitInfo){
//        if (StringUtils.isEmpty(updateCircuitInfo.getTransportationNo())){
//            fb.setCode(0);
//            fb.setMessage("运输编号不能为空");
//            return gson.toJson(fb);
//        }
//        if(circuitService.getByTransportationNo(updateCircuitInfo.getTransportationNo()).getCircuitState()!=0){
//            fb.setCode(0);
//            fb.setMessage("只能在线路未开始运输时才能修改");
//            return gson.toJson(fb);
//        }
//        if(StringUtils.isEmpty(updateCircuitInfo.getSN())){
//            fb.setCode(0);
//            fb.setMessage("网关编号不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getTDLSN())){
//            fb.setCode(0);
//            fb.setMessage("TDL编号不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getAlarmType())){
//            fb.setCode(0);
//            fb.setMessage("报警类型不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getCargoNo())){
//            fb.setCode(0);
//            fb.setMessage("货物编号不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getPlateNo())){
//            fb.setCode(0);
//            fb.setMessage("车牌号不能为空");
//            return gson.toJson(fb);
//        }
//        if (StringUtils.isEmpty(updateCircuitInfo.getCargoName())){
//            fb.setCode(0);
//            fb.setMessage("货物别名不能为空");
//            return gson.toJson(fb);
//        }
//        if(updateCircuitInfo.getAddressVoList().size() ==0){
//            fb.setCode(0);
//            fb.setMessage("起始城市不能为空");
//            return gson.toJson(fb);
//        }
//        if(updateCircuitInfo.getAddressVoList().size()==1){
//            fb.setCode(0);
//            fb.setMessage("目的地不能为空");
//            return gson.toJson(fb);
//        }
//        if(updateCircuitInfo.getAddressVoList().size()>4) {
//            fb.setCode(0);
//            fb.setMessage("目的地不能超过3个");
//            return gson.toJson(fb);
//        }
//        String countryVar="";
//        String cityVar="";
//        String addressDetailVar="";
//        String lngVar="";
//        String latVar = "";
//        String postCodeVar = "";
//        String expTimeVar = "";
//        String compTimeVar = "";
//        String arrivalTimeVar="";
//        String startTimeVar="";
//        Long time = System.currentTimeMillis();
//        for( int i= 0; i<updateCircuitInfo.getAddressVoList().size() ; i++){
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getCountry())){
//                fb.setCode(0);
//                fb.setMessage("国家不能为空");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getCity())){
//                fb.setCode(0);
//                fb.setMessage("城市不能为空");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getAddressDetail())){
//                fb.setCode(0);
//                fb.setMessage("详细地址不能为空");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getLng())){
//                fb.setCode(0);
//                fb.setMessage("经度不能为空");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getLat())){
//                fb.setCode(0);
//                fb.setMessage("纬度不能为空");
//                return gson.toJson(fb);
//            }
//            if(!updateCircuitInfo.getAddressVoList().get(i).getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
//                fb.setCode(0);
//                fb.setMessage("纬度的范围为-90~90");
//                return gson.toJson(fb);
//            }
//            if(!updateCircuitInfo.getAddressVoList().get(i).getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[1-7]\\d)((\\.\\d{1,6})?)|180)$")) {
//                fb.setCode(0);
//                fb.setMessage("经度的范围为-180~180");
//                return gson.toJson(fb);
//            }
//            if (StringUtils.isEmpty(updateCircuitInfo.getAddressVoList().get(i).getPostCode())){
//                fb.setCode(0);
//                fb.setMessage("邮编不能为空");
//                return gson.toJson(fb);
//            }
//            countryVar=countryVar +updateCircuitInfo.getAddressVoList().get(i).getCountry() +" ";
//            cityVar=cityVar +updateCircuitInfo.getAddressVoList().get(i).getCity() +" ";
//            addressDetailVar =addressDetailVar +updateCircuitInfo.getAddressVoList().get(i).getAddressDetail() +" ";
//            lngVar = lngVar + updateCircuitInfo.getAddressVoList().get(i).getLng() +" ";
//            latVar = latVar + updateCircuitInfo.getAddressVoList().get(i).getLat() +" ";
//            postCodeVar=postCodeVar + updateCircuitInfo.getAddressVoList().get(i).getPostCode() +" ";
//            if(updateCircuitInfo.getAddressVoList().get(i).getExpTime() ==null){
//                expTimeVar = expTimeVar + "0 ";
//            }else{
//                expTimeVar = expTimeVar +updateCircuitInfo.getAddressVoList().get(i).getExpTime() +" ";
//            }
//            if(updateCircuitInfo.getAddressVoList().get(i).getCompTime() == null){
//                compTimeVar  = compTimeVar  +"0 ";
//            }else{
//                compTimeVar  = compTimeVar  +updateCircuitInfo.getAddressVoList().get(i).getCompTime() +" ";
//            }
//            if(i ==0){
//                startTimeVar =startTimeVar + time +" ";
//                arrivalTimeVar = arrivalTimeVar +time +" ";
//            }else{
//                startTimeVar =startTimeVar + "0 ";
//                arrivalTimeVar = arrivalTimeVar +"0 ";
//            }
//        }
//        Map<Object,Object> map=new HashMap<>();
//        String transportation = "汽运";
//        String transportationType="本地运输";
//        map.put("transportationNo",updateCircuitInfo.getTransportationNo());
//        map.put("startTime",time);
//        map.put("expTime",updateCircuitInfo.getAddressVoList().get(0).getExpTime());
//        map.put("compTime",updateCircuitInfo.getAddressVoList().get(updateCircuitInfo.getAddressVoList().size()-1).getCompTime());
//        map.put("alarmType",updateCircuitInfo.getAlarmType());
//        map.put("cargoNo",updateCircuitInfo.getCargoNo());
//        map.put("cargoName",updateCircuitInfo.getCargoName());
//        map.put("plateNo",updateCircuitInfo.getPlateNo());
//        map.put("deliveryAddress","站点");
//        map.put("transportation",transportation);
//        map.put("transportationType",transportationType);
//        map.put("createTime",time);
//        map.put("remark",updateCircuitInfo.getRemark());
//        map.put("counts",updateCircuitInfo.getAddressVoList().size());
//        map.put("countryVar",countryVar);
//        map.put("cityVar",cityVar);
//        map.put("addressDetailVar",addressDetailVar);
//        map.put("lngVar",lngVar);
//        map.put("latVar",latVar);
//        map.put("postCodeVar",postCodeVar);
//        map.put("arrivalTimeVar",arrivalTimeVar);
//        map.put("startTimeVar",startTimeVar);
//        map.put("expTimeVar",expTimeVar);
//        map.put("compTimeVar",compTimeVar);
////        map.put("SN",addCircuitVo.getSN());
////        map.put("type",type);
////        map.put("TDLSN",addCircuitVo.getTDLSN());
////        map.put("message",gson.toJson(configCMDVo));
////        map.put("flag",false);
////        map.put("untie",false);
//        circuitService.updateCircuit(map);
//        Map<Object,Object> msg=new HashMap<>();
//        msg.put("msg",map.get("msg"));
//        if (msg.get("msg").equals("修改线路成功")){
//            fb.setCode(1);
//            fb.setMessage("修改线路成功");
//        }else{
//            fb.setCode(0);
//            fb.setMessage(msg.get("msg").toString());
//        }
//        return gson.toJson(fb);
//    }

    //    @ApiOperation(value = "根据线路编号获取线路详细信息",notes = "根据线路编号获取线路详细信息，传值说明：" +
//            "   transportationNo:运输编号；   " +
//            "返回值说明：" +
//            " " )
//    @ApiImplicitParams({
//            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
//    })
//    @PostMapping("/getAlarmData")
//    public Object getAlarmData(@RequestBody String transportationNo){
//        JSONObject jsonObject= JSON.parseObject(transportationNo);
//        transportationNo=(String)jsonObject.get("transportationNo");
//        List<ResultAlarmLog> list= alarmLogService.getByTransportationNo(transportationNo);
//        return  gson.toJson(list);
//    }
}
